pub struct Parser<'a, T>where
T: ParserToken<'a>,{ /* private fields */ }Expand description
Implements a WDL parser.
The parser produces a list of events that can be used to construct a CST.
Implementations
Sourceimpl<'a, T> Parser<'a, T>where
T: ParserToken<'a>,
impl<'a, T> Parser<'a, T>where
T: ParserToken<'a>,
Sourcepub fn peek(&mut self) -> Option<(T, Span)>
pub fn peek(&mut self) -> Option<(T, Span)>
Peeks at the next token (i.e. lookahead 1) from the lexer without consuming it.
The token is not added to the event list.
Note
Note that peeking may cause parser events to be buffered.
If peek returns None, ensure all buffered events are added to the
event list by calling next on the parser; otherwise, calling finish
may panic.
Sourcepub fn peek2(&mut self) -> Option<Peek2<T>>
pub fn peek2(&mut self) -> Option<Peek2<T>>
Peeks at the next and next-next tokens (i.e. lookahead 2) from the lexer without consuming either token.
The returned tokens are not added to the event list.
Sourcepub fn next_if(&mut self, token: T) -> bool
pub fn next_if(&mut self, token: T) -> bool
Consumes the next token only if it matches the given token.
Returns true if the token was consumed, false if otherwise.
Sourcepub fn matching<F>(
&mut self,
open: T,
close: T,
allow_empty: bool,
cb: F,
) -> Result<(), Diagnostic>
pub fn matching<F>( &mut self, open: T, close: T, allow_empty: bool, cb: F, ) -> Result<(), Diagnostic>
Parses a matching token pair that surrounds an item.
This method parses the open token, calls the callback to parse the item, and then parses the close token.
Sourcepub fn matching_delimited<F>(
&mut self,
open: T,
close: T,
delimiter: Option<T>,
recovery: TokenSet,
cb: F,
) -> Result<(), Diagnostic>
pub fn matching_delimited<F>( &mut self, open: T, close: T, delimiter: Option<T>, recovery: TokenSet, cb: F, ) -> Result<(), Diagnostic>
Parses a matching token pair that surround a delimited list of items.
This method parses the open token, calls the callback for each delimited item, and then parses the close token.
The provided recovery token set is used to recover within the delimited item list.
Sourcepub fn consume_close_token(&mut self, open: T, open_span: Span, close: T)
pub fn consume_close_token(&mut self, open: T, open_span: Span, close: T)
Consumes a close token if it is the next token to be parsed.
Otherwise, emits an “unmatched” diagnostic and synthesizes the close token into the parser’s list of events.
Sourcepub fn delimited<F>(
&mut self,
until: T,
delimiter: Option<T>,
recovery: TokenSet,
cb: F,
)
pub fn delimited<F>( &mut self, until: T, delimiter: Option<T>, recovery: TokenSet, cb: F, )
Parses a delimited list of items until the given token.
The provided recovery token set is used to recover within the delimited item list.
The until token is not consumed.
Sourcepub fn diagnostic(&mut self, diagnostic: Diagnostic)
pub fn diagnostic(&mut self, diagnostic: Diagnostic)
Adds a diagnostic to the parser output.
Sourcepub fn push_recovery_set(&mut self, tokens: TokenSet)
pub fn push_recovery_set(&mut self, tokens: TokenSet)
Pushes a token set to the parser’s recovery token set stack.
Sourcepub fn pop_recovery_set(&mut self)
pub fn pop_recovery_set(&mut self)
Pops a token set from the parser’s recovery token set stack.
Panics
Panics if the parser’s recovery set is empty.
Sourcepub fn recover(&mut self, diagnostic: Diagnostic)
pub fn recover(&mut self, diagnostic: Diagnostic)
Recovers from an error by consuming all tokens not in the top-most recovery set.
Panics
Panics if a recovery set was not pushed with Self::push_recovery_set.
Sourcepub fn recover_with_set(&mut self, diagnostic: Diagnostic, recovery: TokenSet)
pub fn recover_with_set(&mut self, diagnostic: Diagnostic, recovery: TokenSet)
Performs recovery with the given recovery token set.
Sourcepub fn require(&mut self, token: T) -> Span
pub fn require(&mut self, token: T) -> Span
Requires that the current token is the given token.
Panics if the token is not the given token.
Sourcepub fn require_in(&mut self, tokens: TokenSet)
pub fn require_in(&mut self, tokens: TokenSet)
Requires that the current token is in the given token set.
Panics
Panics if the token is not in the token set.
Sourcepub fn expect(&mut self, token: T) -> Result<Span, Diagnostic>
pub fn expect(&mut self, token: T) -> Result<Span, Diagnostic>
Expects the next token to be the given token.
Returns an error if the token is not the given token.
Sourcepub fn expect_with_name(
&mut self,
token: T,
name: &'static str,
) -> Result<Span, Diagnostic>
pub fn expect_with_name( &mut self, token: T, name: &'static str, ) -> Result<Span, Diagnostic>
Expects the next token to be the given token, but uses the provided name in the error.
Returns an error if the token is not the given token.
Sourcepub fn expect_in(
&mut self,
tokens: TokenSet,
expected: &[&str],
) -> Result<(T, Span), Diagnostic>
pub fn expect_in( &mut self, tokens: TokenSet, expected: &[&str], ) -> Result<(T, Span), Diagnostic>
Expects the next token to be in the given token set.
Returns an error if the token is not the given set.
Sourcepub fn interpolate<T2, F, R>(&mut self, cb: F) -> R
pub fn interpolate<T2, F, R>(&mut self, cb: F) -> R
Used to interpolate strings with a different string interpolation token.
The provided callback receives a Interpolator.
The callback should use Interpolator::into_parser for the return value.
Sourcepub fn morph<T2>(self) -> Parser<'a, T2> ⓘ
pub fn morph<T2>(self) -> Parser<'a, T2> ⓘ
Morph this parser into a parser for a new token type.
The returned parser continues to point at the same span as the current parser.
Sourcepub fn into_interpolator<T2>(self) -> Interpolator<'a, T2> ⓘ
pub fn into_interpolator<T2>(self) -> Interpolator<'a, T2> ⓘ
Consumes the parser and returns an interpolator.
Sourcepub fn finish(self) -> Output<'a, T>
pub fn finish(self) -> Output<'a, T>
Consumes the parser and returns the output.
Panics
This method panics if buffered events remain in the parser.
To ensure that no buffered events remain, call next() on the parser
and verify it returns None before calling this method.
Sourcepub fn update_last_token_kind(&mut self, new_kind: SyntaxKind)
pub fn update_last_token_kind(&mut self, new_kind: SyntaxKind)
Sourcepub fn consume_remainder(&mut self)
pub fn consume_remainder(&mut self)
Consumes the remainder of the unparsed source into a special “unparsed” token.
This occurs when a source file is missing a version statement or if the version specified is unsupported.